'use strict';

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.debounce = debounce;
exports.asyncHandlerLockWrap = asyncHandlerLockWrap;
exports.getDeepValue = getDeepValue;

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * 针对本地开发 console.log 添加颜色打印
 */
/*
export const $log = (function() {
  const isDEV = process.env.NODE_ENV !== 'production' // 开发环境
  if (!isDEV) {
    // eslint-disable-next-line
    // @ts-ignore
    return (content = 'content', title = 'title', background = '#007bfc') => {}
  }
  return (content = 'content', title = 'title', background = '#007bfc') => {
    content = typeof content === 'object' ? JSON.stringify(content) : content
    console.log.apply(null, [`%c ${title}： %c ${content} `, 'padding: 1px; border-radius: 10px; color: #fff; background: #9159B2;', `padding: 1px; border-radius: 10px; color: #fff; background: ${background};`])
  }
})()*/
var $log = exports.$log = function $log() {
  var content = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'content';
  var title = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'title';
  var background = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '#007bfc';

  content = (typeof content === 'undefined' ? 'undefined' : _typeof(content)) === 'object' ? JSON.stringify(content) : content;
  console.log.apply(null, ['%c ' + title + '\uFF1A %c ' + content + ' ', 'padding: 1px; border-radius: 10px; color: #fff; background: #9159B2;', 'padding: 1px; border-radius: 10px; color: #fff; background: ' + background + ';']);
};

/**
 * @desc 函数防抖
 * @param func 函数
 * @param wait 延迟毫秒数
 * @param immediate true/false (是/否)即执行
 */
function debounce(func, wait, immediate) {
  var timeout = void 0;

  return function () {
    /* eslint-disable */
    // @ts-ignore
    var context = this;
    var args = arguments;
    /* eslint-enable */
    if (timeout) clearTimeout(timeout);
    if (immediate) {
      var callNow = !timeout;
      timeout = setTimeout(function () {
        timeout = null;
      }, wait);
      if (callNow) func.apply(context, args);
    } else {
      timeout = setTimeout(function () {
        func.apply(context, args);
      }, wait);
    }
  };
}

/**
 * 深层数据的集合进行有机合并
 * @param objs[] 需要合并的 对象集合 （后面的覆盖前面的参数）
 * @returns {{}}
 */
var objDeepMerge = exports.objDeepMerge = function objDeepMerge() {
  for (var _len = arguments.length, objs = Array(_len), _key = 0; _key < _len; _key++) {
    objs[_key] = arguments[_key];
  }

  var res = {};
  var combine = function combine(opt) {
    for (var prop in opt) {
      // eslint-disable-next-line
      if (opt.hasOwnProperty(prop)) {
        if (Object.prototype.toString.call(opt[prop]) === '[object Object]') {
          res[prop] = objDeepMerge(res[prop], opt[prop]);
        } else {
          res[prop] = opt[prop];
        }
      }
    }
  };
  objs.map(function (opts) {
    combine(opts);
  });
  return res;
};

/**
 * 异步操作避免重复触发加锁
 * @param syncFn
 * @returns {(function(...[*]): Promise<void>)|*}
 */
function asyncHandlerLockWrap(syncFn) {
  var is_lock = false;
  /* eslint-disable */
  // @ts-ignore
  return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var context,
        _len2,
        args,
        _key2,
        _args = arguments;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // @ts-ignore
            context = this;

            if (!is_lock) {
              is_lock = true;
              // @ts-ignore

              for (_len2 = _args.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = _args[_key2];
              }

              Promise.resolve().then(syncFn.bind.apply(syncFn, [context].concat(args))).finally(function () {
                is_lock = false;
              });
            }

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  /* eslint-enable */
}

/**
 * 提取深层数据的值 (防止中间项 不存在导致的报错)
 * @param obj 【检测对象】
 * @param keyArr 【需要获取该对象内部数据key数组】
 * @returns {*}
 */
function getDeepValue(obj, keyArr) {
  return keyArr.reduce(function (acc, key) {
    return acc && acc[key];
  }, obj);
}